import sys
import modulemd
import logging
from . import _depchase, _repodata

def _name_only(rpm_name):
    name, version, release = rpm_name.rsplit("-", 2)
    return name

def _categorise_deps(all_rpm_deps):
    module_deps = set()
    remaining_rpm_deps = set()
    for pkgname in all_rpm_deps:
        modname = _repodata.get_module_for_rpm(pkgname)
        if modname is not None:
            module_deps.add(modname)
        else:
            remaining_rpm_deps.add(pkgname)
    return module_deps, remaining_rpm_deps

class ModuleGenerator(object):

    def __init__(self, pkgs):
        self.pkgs = pkgs
        self.core_srpm = None
        self.mmd = modulemd.ModuleMetadata()
        self._pool = _depchase.make_pool("x86_64")

    def _calculate_dependencies(self):
        pkgs = self.pkgs
        pool = self._pool
        self.api_srpms = {_name_only(_depchase.get_srpm_for_rpm(pool, dep)) for dep in pkgs}
        run_deps = _depchase.ensure_installable(pool, pkgs)
        module_run_deps, rpm_run_deps = _categorise_deps(run_deps)
        # The platform module provides any other runtime dependencies - we expect we'll
        # always depend on it, but check for the heck of it.
        if len(rpm_run_deps) > 0:
            module_run_deps.add('platform')
        self.module_run_deps = module_run_deps
        self.run_srpms = set()

    def _set_core_srpm(self):
        """Set core SRPM based on first listed package"""
        pool = self._pool
        pkg = self.pkgs[0]
        self.core_srpm = _name_only(_depchase.get_srpm_for_rpm(pool, pkg))

    def _save_module_md(self, output_fname):
        """
        Function writes modulemd to either stdout or the given filename
        :return:
        """
        if output_fname is not None:
            self.mmd.dump(output_fname)
            print('Generated modulemd file: %r' % output_fname)
        else:
            print(self.mmd.dumps())
        return True

    def _update_module_md(self):
        """
        Function updates modulemd file with dependencies
        are information taken from SPEC file.
        :return:
        """
        self.mmd.add_module_license("MIT")

        if self.core_srpm is not None:
            self.mmd.summary = f"Generated module for {self.core_srpm}"
        else:
            self.mmd.summary = f"Generated module for {self.pkgs}"

        self.mmd.description = "Module auto-generated by fedmod"

        # Declare the public API
        for pkg in self.api_srpms:
            self.mmd.api.add_rpm(pkg)
            self.mmd.components.add_rpm(pkg, "Package in api", buildorder=self._get_build_order(pkg))

        # Declare module level dependencies
        for modname in self.module_run_deps:
            self.mmd.buildrequires[modname] = "f28"
            self.mmd.requires[modname] = "f28"

        for pkg in self.run_srpms:
            self.mmd.components.add_rpm(pkg, "Runtime dependency.", buildorder=self._get_build_order(pkg))

        # TODO: Always set content licenses appropriately

        # TODO: Emit something for non-empty self.unresolved_build_rpms
        #       rather than relying solely on the warnings emitted on stderr


    def _get_build_order(self, pkg):
        if pkg in self.mmd.api.rpms:
            return 10
        else:
            return 0

    def run(self, output_fname):
        if len(self.pkgs) == 1:
            self._set_core_srpm()
        self._calculate_dependencies()
        self._update_module_md()
        self._save_module_md(output_fname)
